#!/usr/bin/python

from socket import gethostname
import os
import sys
import pwd
import grp
import subprocess
import shutil
import tempfile

local_copy = os.path.join(
    os.path.dirname(os.path.abspath(os.path.dirname(__file__))),
    "lib")
if os.path.exists(local_copy) and os.path.isdir(local_copy):
    sys.path.insert(0, local_copy)

from charmhelpers.fetch import (
    apt_install,
)

from charmhelpers.core.host import (
    mkdir,
    service,
    service_start,
    service_stop,
    write_file,
)

from charmhelpers.core.hookenv import (
    Hooks,
    config,
    open_port,
    relation_get,
    relation_id,
    remote_unit,
    log,
)

hook_dir = os.path.abspath(os.path.dirname(__file__))
charm_dir = os.path.dirname(hook_dir)

host = gethostname()

sudoers_str = config().get('sudoers_groups')
sudoers_groups = sudoers_str.split()
template_host = config().get('template_host')
ssh_user = config().get('ssh_user')

templatefile = "/var/lib/misc/%s" % template_host
hostfile = "/var/lib/misc/%s" % host
sshuser_home = "/home/%s" % ssh_user
sshkeys_home = "%s/.ssh" % sshuser_home
user_auth_keys = "/etc/ssh/user-authorized-keys"
cache_dir = "/var/cache/userdir-ldap/hosts"
files_dir = os.path.join(os.environ['CHARM_DIR'], 'files')

if os.path.isdir(user_auth_keys):
    authorized_keys = "%s/%s" % (user_auth_keys, ssh_user)
else:
    authorized_keys = "%s/authorized_keys" % sshkeys_home

required_pkgs = [
    'python-jinja2',
]


def getuser(name):
    try:
        return pwd.getpwnam(name)
    except KeyError:
        return None


def getgroup(name):
    try:
        return grp.getgrnam(name)
    except KeyError:
        return None


hooks = Hooks()


def update_config():
    print "Putting sudoers in place"
    # Put sudoers in place
    from jinja2 import Environment, FileSystemLoader

    template_env = Environment(
        loader=FileSystemLoader(os.path.join(os.environ['CHARM_DIR'],
                                'templates')))
    templ_vars = {
        'ssh_user': ssh_user,
        'sudoers_groups': sudoers_groups,
        'template_host': template_host,
    }
    template = template_env.get_template('sudoers/50-bootstack').render(templ_vars)
    sudoers_file = "/etc/sudoers.d/50-bootstack"
    with open(sudoers_file, 'w') as sudoers_file_config:
        sudoers_file_config.write(str(template))

    if not os.path.isdir(cache_dir):
        os.makedirs(cache_dir)

    template = template_env.get_template('cron/bootstack-config.tmpl').render(templ_vars)
    cron_file = "/etc/cron.d/bootstack-config"
    with open(cron_file, 'w') as cron_file_config:
        cron_file_config.write(str(template))

    print "Fixing userdir-ldap host dir"
    # Fix up ud-ldap host dir
    if os.path.isfile(templatefile) and not os.path.islink(hostfile):
        os.symlink(templatefile, hostfile)

    print "Checking sshdist user"
    # If sshdist user doesn't exist, create it.
    if not getuser(ssh_user):
        print "sshdist user doesn't exist"
        if getgroup(ssh_user):
            cmd = "/usr/sbin/useradd -c \"%s\" -s /bin/bash -m -d /home/%s -g %s %s" % (ssh_user, ssh_user, ssh_user, ssh_user)
        else:
            cmd = "/usr/sbin/useradd -c \"%s\" -s /bin/bash -m -d /home/%s %s" % (ssh_user, ssh_user, ssh_user)
        # try:
        output = subprocess.check_output(cmd.split(), stderr=subprocess.STDOUT)
        print "Output is %s" % output
        # except subprocess.CalledProcessError:
        #    print "CalledProcessError"
        #    pass
        # except OSError:
        #    print "OSError"
        #    pass

    # Get the uid and gid for later, now that the user should exist
    ssh_uid = pwd.getpwnam(ssh_user).pw_uid
    ssh_gid = grp.getgrnam(ssh_user).gr_gid
    # Make sure the users homedir is owned correctly
    os.chown(sshuser_home, ssh_uid, ssh_gid)

    print "Checking ssh directory"
    # Make .ssh dir if it doesn't exist
    if not os.path.isdir(sshkeys_home):
        os.makedirs(sshkeys_home, 0750)
        os.chown(sshkeys_home, ssh_uid, ssh_gid)

    print "Fixing up authorized_keys"
    # Bundle up the authorized_keys file
    sshkeys_path = os.path.join(files_dir, "sshkeys")
    with tempfile.NamedTemporaryFile(delete=False) as temp:
        print "temp file is %s" % temp.name
        for fil in os.listdir(sshkeys_path):
            fpath = os.path.join(sshkeys_path, fil)
            if os.path.isfile(fpath):
                print "%s is a file" % fpath
                with open(fpath) as f:
                    lines = f.readlines()
                    print "%s read: " % lines
                    for l in lines:
                        temp.write(str(l))
                f.close()
        temp.close()
        shutil.copyfile(temp.name, authorized_keys)
        os.chmod(authorized_keys, 0600)
        os.chown(authorized_keys, ssh_uid, ssh_gid)


@hooks.hook('install', 'config-changed', 'upgrade-charm')
def install():
    apt_install(required_pkgs, options=['--force-yes'])
    update_config()


if __name__ == "__main__":
    hooks.execute(sys.argv)
